// tiedbin/full-to-tied.cc

// Copyright 2011 Univ. Erlangen-Nuremberg, Korbinian Riedhammer

// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//  http://www.apache.org/licenses/LICENSE-2.0
//
// THIS CODE IS PROVIDED *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
// WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
// MERCHANTABLITY OR NON-INFRINGEMENT.
// See the Apache 2 License for the specific language governing permissions and
// limitations under the License.


#include "base/kaldi-common.h"
#include "util/common-utils.h"
#include "gmm/diag-gmm.h"
#include "gmm/full-gmm.h"

int main(int argc, char *argv[]) {
  try {
    using namespace kaldi;
    using kaldi::int32;

    const char *usage =
        "Convert a full covariance GMM into a diagonal one.\n"
        "Usage: full-to-tied <full-gmm-in> <diag-gmm-out>\n";

    bool binary = false;
    ParseOptions po(usage);
    po.Register("binary", &binary, "Write output in binary mode");
    po.Read(argc, argv);

    if (po.NumArgs() != 2) {
      po.PrintUsage();
      exit(1);
    }

    std::string fin  = po.GetArg(1);
    std::string fout = po.GetArg(2);

    FullGmm full;
    bool binary_in;
    Input ki1(fin, &binary_in);
    full.Read(ki1.Stream(), binary_in);

    DiagGmm diag;
    diag.CopyFromFullGmm(full);

    Output ou1(fout, binary);
    diag.Write(ou1.Stream(), binary);

    return 0;
  } catch(const std::exception& e) {
    std::cerr << e.what();
    return -1;
  }
}

